<?php
// $Id: cmis_alfresco.cmis.inc,v 1.3 2009/09/09 16:51:41 cbalan Exp $

/**
 * Implementation of cmisapi_getRepositoryInfo method 
 *
 */
function cmis_alfresco_cmisapi_getRepositoryInfo() {
  static $repository_info;
  
  if (empty($repository_info)) {
    module_load_include('utils.inc', 'cmis_alfresco');
  
    $response = cmis_alfresco_invoke_service('/api/repository');
    $xml = cmis_alfresco_utils_get_CMIS_xml($response);
    
    if ($xml) {
      $cmis = $xml->workspace->children(cmis_alfresco_utils_ns('cmis'));
      $repository_info = $cmis->repositoryInfo;
    } 
  }
  return $repository_info;
}

/**
 * Implementation of cmisapi_getTypes method
 * 
 * @param $repositoryId
 * @param $typeId 
 */
function cmis_alfresco_cmisapi_getTypes($repositoryId, $typeId = NULL) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $service = $typeId?'/api/type/'. $typeId .'/descendants':'/api/types';
  $response = cmis_alfresco_invoke_service($service);
  
  if (false != $response) {
    $items = array();
    foreach (cmis_alfresco_utils_get_CMIS_xml($response, '//D:entry') as $entry) {
      $item = _cmis_alfresco_utils_entry($entry);
      // fixing typeId, since  _cmis_alfresco_utils_entry will try to resolve type's id as a document id
      $item->id = (string) $entry->content;
      $items[] = $item;
    }

    return $items;
  }
  
  return FALSE;
}

/**
 * Implementation of cmisapi_getTypeDefinition
 * 
 * @param $repositoryId
 * @param $typeId
 */
function cmis_alfresco_cmisapi_getTypeDefinition($repositoryId, $typeId) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $response = cmis_alfresco_invoke_service('/api/type/'. $typeId);
  
  if (false != $response) {
    $xml = cmis_alfresco_utils_get_CMIS_xml($response);
    $cmis_type = _cmis_alfresco_utils_entry($xml);
    
    $cmis_element = $xml->children(cmis_alfresco_utils_ns('cmis'));

    $properties_element = NULL;
    if ($cmis_element->documentType) {
      $properties_element = $cmis_element->documentType;
    } 
    elseif ($cmis_element->folderType) {
      $properties_element = $cmis_element->folderType;
    }
    
    if ($properties_element) {
      foreach (_cmis_alfresco_utils_known_property_types() as $type) {
        $property_tag = 'property'. $type .'Definition';
        foreach ($properties_element->$property_tag as $property) {
          $attrs = $property->attributes(cmis_alfresco_utils_ns('cmis'));
          $cmis_type->fields[ (string) $property->name ] = array(
            'id' => (string) $property->id, 
            'name' => (string) $property->name,
            'displayName' => (string) $property->displayName,
            'propertyType' => (string) $property->propertyType
          );
        }
      }
    }
    
    return $cmis_type;
  }
  
  return FALSE;
}

/**
 * Implementation of cmisapi_getChildren method
 * 
 * @param $repositoryId
 * @param $folderId
 */
function cmis_alfresco_cmisapi_getChildren($repositoryId, $folderId) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $folderId = cmis_alfresco_objectId($folderId);
  if ($folderId['noderef_url']) {
    $url = '/api/node/'. $folderId['noderef_url'];
  }
  else {
    $url = $folderId['url'];
  }

  $response = cmis_alfresco_invoke_service($url .'/children');
  if (false != $response) {
    return _cmis_alfresco_getEntries(cmis_alfresco_utils_get_CMIS_xml($response, '//D:entry'));
  }

  return FALSE;
}

/**
 * Implementation of cmisapi_getObjectParents method
 * 
 * @param $repositoryId
 * @param $folderId
 */
function cmis_alfresco_cmisapi_getObjectParents($repositoryId, $folderId) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $folderId = cmis_alfresco_objectId($folderId);
  if ($folderId['noderef_url']) {
    $url = '/api/node/'. $folderId['noderef_url'];
  }
  else {
    $url = $folderId['url'];
  }

  $response = cmis_alfresco_invoke_service($url .'/parent');
  if (false != $response) {
    return _cmis_alfresco_getEntries(cmis_alfresco_utils_get_CMIS_xml($response, '//D:entry'));
  }

  return FALSE;
}

/**
 * Implementation of cmisapi_createDocument method 
 * 
 * @param $repositoryId
 * @param $objectTypeId
 * @param $properties
 * @param $folderId
 * @param $content
 * @param $versioningState
 */
function cmis_alfresco_cmisapi_createDocument($repositoryId, $objectTypeId = 'document', $properties = array(), $folderId = NULL, $content = NULL, $versioningState = NULL) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $parentFolderId = cmis_alfresco_objectId($folderId);
  if (!is_array($parentFolderId) || !array_key_exists('noderef_url', $parentFolderId)) {
    throw new CMISException('Unable to find destination folder: '. $folderId);
  }

  $title = $properties['title'];
  unset($properties['title']);
  
  $summary = $properties['summary'];
  unset($properties['summary']);
  
  $content_type = $properties['content-type'];
  unset($properties['content-type']);
  
  $properties['ObjectTypeId'] = $objectTypeId;
  $properties_xml = _cmis_alfresco_utils_properties_to_xml($properties);  
    
  $postvars = '<?xml version="1.0" encoding="utf-8"?>'.
  '<entry xmlns="http://www.w3.org/2005/Atom" xmlns:cmis="'. cmis_alfresco_utils_ns('cmis') .'">'.
  '<title>'. $title .'</title>'.
  '<summary>'. $summary .'</summary>'.
  '<content type="'. $content_type .'">'. base64_encode($content) .'</content>'.
  '<cmis:object>'.
  '<cmis:properties>'. $properties_xml .'</cmis:properties>'.
  '</cmis:object>'.
  '</entry>';
  
  $header[] = 'Content-type: application/atom+xml;type=entry';
  $header[] = 'Content-length: '. strlen($postvars);
  $header[] = 'MIME-Version: 1.0';

  $response = cmis_alfresco_invoke_service('/api/node/'. $parentFolderId['noderef_url'] .'/children', $header, 'CUSTOM-POST', $postvars);
  $entry = cmis_alfresco_utils_get_CMIS_xml($response, '/D:entry');
  $entry = $entry[0][0];

  $objectId = cmis_alfresco_objectId((string)$entry->id);

  return $objectId['noderef'];
}

/**
 * Implementation of cmisapi_createFolder method 
 * 
 * @param $repositoryId
 * @param $objectTypeId
 * @param $properties
 * @param $folderId
 */
function cmis_alfresco_cmisapi_createFolder($repositoryId, $objectTypeId = 'folder', $properties = array(), $folderId = NULL) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $parentFolderId = cmis_alfresco_objectId($folderId);
  if (!is_array($parentFolderId) || !array_key_exists('noderef_url', $parentFolderId)) {
    throw new CMISException('Unable to find destination folder: '. $folderId);
  }

  $title = $properties['title'];
  unset($properties['title']);
   
  $properties['ObjectTypeId'] = $objectTypeId;
  $properties_xml = _cmis_alfresco_utils_properties_to_xml($properties);  
  
  $postvars = '<?xml version="1.0" encoding="utf-8"?>'.
  '<entry xmlns="http://www.w3.org/2005/Atom" xmlns:cmis="'. cmis_alfresco_utils_ns('cmis') .'">'.
  '<title>'. $title .'</title>'.
  '<cmis:object>'.
  '<cmis:properties>'. $properties_xml .'</cmis:properties>'.
  '</cmis:object>'.
  '</entry>';
  
  $header[] = 'Content-type: application/atom+xml;type=entry';
  $header[] = 'Content-length: '. strlen($postvars);
  $header[] = 'MIME-Version: 1.0';
  
  $response = cmis_alfresco_invoke_service('/api/node/'. $parentFolderId['noderef_url'] .'/children', $header, 'CUSTOM-POST', $postvars);
  $entry = cmis_alfresco_utils_get_CMIS_xml($response, '/D:entry');
  $entry = $entry[0][0];

  $objectId = cmis_alfresco_objectId((string)$entry->id);

  return $objectId['noderef'];
}

/**
 * Implementation of cmisapi_getProperties method
 * 
 * @param $repositoryId
 * @param $objectId
 */
function cmis_alfresco_cmisapi_getProperties($repositoryId, $objectId) {
  module_load_include('utils.inc', 'cmis_alfresco');
  $objectId = cmis_alfresco_objectId($objectId);

  if ($response = cmis_alfresco_invoke_service('/api/node/'. $objectId['noderef_url'])) {
    if ($entries = _cmis_alfresco_getEntries(cmis_alfresco_utils_get_CMIS_xml($response, '//D:entry'))) {
      return $entries[0];
    }
  }

  watchdog('cmis_alfresco_cmisapi_getProperties', 'Unknown objectId "@objectId"', array('@objectId' => $objectId));
  return FALSE;
}

/**
 * Implementation of cmisapi_updateProperties method
 * 
 * @param $repositoryId
 * @param $objectId
 * @param $changeToken
 * @param $propertyCollection
 */
function cmis_alfresco_cmisapi_updateProperties($repositoryId, $objectId, $changeToken = NULL, $propertyCollection = array()) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $alfresco_objectId = cmis_alfresco_objectId($objectId);
  if (!is_array($alfresco_objectId) || !array_key_exists('noderef_url', $alfresco_objectId)) {
    drupal_set_message('Unable to find destination object: '. $objectId, 'error');
    return FALSE;
  }

  $properties_xml = _cmis_alfresco_utils_properties_to_xml($propertyCollection);
  
  $postvars = '<?xml version="1.0" encoding="utf-8"?>'.
  '<entry xmlns="http://www.w3.org/2005/Atom" xmlns:cmis="'. cmis_alfresco_utils_ns('cmis') .'">'.   
  '<cmis:object>'.
  '<cmis:properties>'. $properties_xml .'</cmis:properties>'.
  '</cmis:object>'.
  '</entry>';
  
  $header[] = 'Content-type: application/atom+xml;type=entry';
  $header[] = 'Content-length: '. strlen($postvars);
  $header[] = 'MIME-Version: 1.0';

  $response = cmis_alfresco_invoke_service('/api/node/'. $alfresco_objectId['noderef_url'], $header, 'CUSTOM-PUT', $postvars);
  $entry = cmis_alfresco_utils_get_CMIS_xml($response, '/D:entry');
  $entry = $entry[0][0];

  $result = cmis_alfresco_objectId((string) $entry->id);

  return $result['noderef'];
}

/**
 * Implemetation of cmisapi_getContentStream method
 * 
 * @param $repositoryId
 * @param $objectId
 */
function cmis_alfresco_cmisapi_getContentStream($repositoryId, $objectId) {
  module_load_include('utils.inc', 'cmis_alfresco');
  
  $objectId = cmis_alfresco_objectId($objectId);
  return cmis_alfresco_invoke_service('/api/node/content/'. $objectId['noderef_url']);
}

/**
 * Implementation of cmisapi_setContentStream method
 * 
 * @param $repositoryId
 * @param $objectId
 * @param $overwriteFlag
 * @param $content
 * @param $properties
 * 
 * @todo 
 *   send all known properties, see cmis_alfresco_cmisapi_updateProperties
 */
function cmis_alfresco_cmisapi_setContentStream($repositoryId, $objectId, $overwriteFlag = TRUE, $content = NULL, $properties = array()) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $objectId = cmis_alfresco_objectId($objectId);
  $properties_xml = _cmis_alfresco_utils_properties_to_xml($properties);  
  
  $postvars = '<?xml version="1.0" encoding="utf-8"?>'.
  '<entry xmlns="http://www.w3.org/2005/Atom" xmlns:cmis="'. cmis_alfresco_utils_ns('cmis') .'">'.
  '<title>'. $properties['title'] .'</title>'.
  '<content type="'. $properties['content-type'] .'">'. $content .'</content>'.
  '<cmis:object>'.
  '<cmis:properties>'. $properties_xml .'</cmis:properties>'.
  '</cmis:object>'.
  '</entry>';

  $header[] = 'Content-type: application/atom+xml;type=entry';
  $header[] = 'Content-length: '. strlen($postvars);
  $header[] = 'MIME-Version: 1.0';

  $objectId = cmis_alfresco_objectId($objectId);
  $response = cmis_alfresco_invoke_service('/api/node/'. $objectId['noderef_url'], $header, 'CUSTOM-PUT', $postvars);
  return $response;
}

/**
 * Implementation of cmisapi_query method
 *   
 * @param $repositoryId
 * @param $statement
 * @param $searchAllVersions
 * @param $includeAllAllowableActions
 * @param $includeRelationships
 * @param $maxItems
 * @param $skipCount
 * @return array
 */
function cmis_alfresco_cmisapi_query($repositoryId, $statement, $searchAllVersions = FALSE, $includeAllAllowableActions = FALSE, $includeRelationships = NULL, $maxItems = 0, $skipCount = 0) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $postvars = '<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>'.
  '<query xmlns="'. cmis_alfresco_utils_ns('cmis') .'">'.
  '<statement>'. $statement .'</statement>'.
  '<searchAllVersions>'. ($searchAllVersions?'true':'false') .'</searchAllVersions>'.
  '<maxItems>'. $maxItems .'</maxItems>'.
  '<skipCount>'. $skipCount .'</skipCount>'.
  '<returnAllowableActions>'. ($includeAllAllowableActions?'true':'false') .'</returnAllowableActions>'.
  '</query>';

  $header[] = 'Content-type: application/cmisquery+xml';
  $header[] = 'Content-length: '. strlen($postvars);
  $header[] = 'MIME-Version: 1.0';

  $response = cmis_alfresco_invoke_service('/api/query', $header, 'CUSTOM-POST', $postvars);
  if (false != $response) {
    return _cmis_alfresco_getEntries(cmis_alfresco_utils_get_CMIS_xml($response, '//D:entry'));
  }

  return FALSE;
}

/**
 * Implementation of cmisapi_deleteObject method
 * 
 * @param $repositoryId
 * @param $objectId
 */
function cmis_alfresco_cmisapi_deleteObject($repositoryId, $objectId) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $objectId = cmis_alfresco_objectId($objectId);  
  $header[] = 'MIME-Version: 1.0';

  $response = cmis_alfresco_invoke_service('/api/node/'. $objectId['noderef_url'], $header, 'DELETE', '');
}